@@ -51,7 +51,7 @@ extern int fdisk_list_table;
 #define _(String) dgettext (PACKAGE, String)
 
 /* BEGIN readline stuff */
-#if HAVE_READLINE
+#if HAVE_LIBREADLINE
 
 /* Termcap need this buffer to initialize terminal database, I think
    the db is stored into this buffer but I'm not sure. */
@@ -66,7 +66,9 @@ static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
 # if HAVE_TERMCAP_H
 #  include <termcap.h>
 # else
+#  if HAVE_LIBTERMCAP
     extern int tgetnum (char* key);
+#  endif
 # endif /* HAVE_TERMCAP_H */
 
 /* readline/readline.h */
@@ -93,7 +95,7 @@ static enum { TERMCAP_OK, TERMCAP_ERROR } termcap_status;
 #  define rl_compentry_func_t void
 # endif
 
-#endif /* HAVE_READLINE */
+#endif /* HAVE_LIBREADLINE */
 
 /* END readline stuff */
 
@@ -127,7 +129,7 @@ static PedExceptionOption current_exception_opt = 0;
 static prompt_possibilities = 1;
 
 
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE && HAVE_LIBTERMCAP
 /* This function initialize the termcap library */
 void init_termcap_library(void)
 {
@@ -189,7 +191,6 @@ fdisk_complete_function (char* text, int start, int end)
   return rl_completion_matches (text, (rl_compentry_func_t*) command_generator);
 }
 
-#if HAVE_READLINE_HISTORY_H
 static void
 _add_history_unique (const char* line)
 {
@@ -215,9 +216,8 @@ _add_history_unique (const char* line)
   /* no match */
   add_history(line);
 }
-#endif /* HAVE_READLINE_HISTORY_H */
 
-#endif /* HAVE_READLINE */
+#endif /* HAVE_LIBREADLINE && HAVE_LIBTERMCAP_H */
 
 static void
 interrupt_handler (int signum)
@@ -259,7 +259,7 @@ _readline (const char* prompt, const StrList* possibilities)
   fdisk_wipe_line ();
 
   if (!fdisk_opt_script_mode)
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE
     { /* !fdisk_opt_script_mode && HAVE_READLINE */
       line = readline(prompt);
       if(line && *line)
@@ -275,6 +275,12 @@ _readline (const char* prompt, const StrList* possibilities)
 	  free (line);
 	  line = NULL;
 	}
+      else
+	{
+	  char *end = strchr(line, 0x0a);
+	  if(end)
+	    *end = 0;
+	}
     }
 #endif
   /* MMM... I assume that `disk_opt_script_mode' is true when fdisk
@@ -289,6 +295,12 @@ _readline (const char* prompt, const StrList* possibilities)
 	  free (line);
 	  line = NULL;
 	}
+      else
+	{
+	  char *end = strchr(line, 0x0a);
+	  if(end)
+	    *end = 0;
+	}
     }
   readline_state.in_readline = 0;
   return line;
@@ -845,7 +857,7 @@ fdisk_screen_width ()
     return 32768;	/* no wrapping ;) */
 
 /* HACK: don't specify termcap separately - it'll annoy the users. */
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE
   if(termcap_status == TERMCAP_OK)
     width = tgetnum ("co");
 #endif
@@ -1437,12 +1449,12 @@ fdisk_init_ui ()
     return 0;
   ped_exception_set_handler (exception_handler);
 
-#ifdef HAVE_READLINE
+#if HAVE_LIBREADLINE
   rl_initialize ();
   rl_attempted_completion_function = (CPPFunction*) fdisk_complete_function;
-  readline_state.in_readline = 0;
-#endif /* HAVE_READLINE */
+#endif /* HAVE_LIBREADLINE */
 
+  readline_state.in_readline = 0; 
   signal (SIGINT, &interrupt_handler);
 
   return 1;
